/*
 * Copyright (C) 2002 Roman Zippel <zippel@linux-m68k.org>
 * Released under the terms of the GNU GPL v2.0.
 */

#include <stdio.h>
#include <string.h>
#include "zconf.tab.h"
#include "zconf.h"

int readconf(void)
{
	FILE *in;
	char line[128], *p, *p2;
	int lineno = 0;
	struct symbol *sym;
	struct statement *stmt;

	in = fopen(".config", "r");
	if (!in) {
		sym = lookup_symbol("ARCH");
		for (stmt = sym->stmt; stmt; stmt = stmt->next)
			if (!stmt->prompt && stmt->def)
				break;
		if (stmt) {
			sprintf(line, "arch/%s/defconfig", stmt->def->name);
			in = fopen(line, "r");
			if (in)
				printf("#\n"
				       "# using defaults found in %s\n"
				       "#\n", line);
		}
	} else
		printf("#\n"
		       "# using defaults found in .config\n"
		       "#\n");
	if (!in)
		return 1;

	while (fgets(line, 128, in)) {
		lineno++;
		switch (line[0]) {
		case '#':
			if (memcmp(line + 2, "CONFIG_", 7))
				continue;
			p = strchr(line + 9, ' ');
			if (!p)
				continue;
			*p++ = 0;
			if (strncmp(p, "is not set", 10))
				continue;
			//printf("%s -> n\n", line + 9);
			sym = lookup_symbol(line + 9);
			switch (sym->type) {
			case SYMBOL_BOOLEAN:
			case SYMBOL_TRISTATE:
				sym->def_value = &symbol_no;
				break;
			}
			break;
		case 'C':
			if (memcmp(line, "CONFIG_", 7))
				continue;
			p = strchr(line + 7, '=');
			if (!p)
				continue;
			*p++ = 0;
			p2 = strchr(p, '\n');
			if (p2)
				*p2 = 0;
			//printf("%s -> %s\n", line + 7, p);
			sym = lookup_symbol(line + 7);
			switch (sym->type) {
			case SYMBOL_BOOLEAN:
				sym->def_value = &symbol_yes;
				break;
			case SYMBOL_TRISTATE:
				if (p[0] == 'm')
					sym->def_value = &symbol_mod;
				else
					sym->def_value = &symbol_yes;
				break;
			case SYMBOL_STRING:
				if (p[0] == '"' && p[strlen(p)-1] == '"') {
					// fix me
					p++;
					p[strlen(p)-1] = 0;
				}
			case SYMBOL_INT:
			case SYMBOL_HEX:
				sym->def_value = alloc_symbol(strdup(p));
				break;
			}
			//if (sym->flags & SYMBOL_CHOICE) {
			//	for (stmt = sym->stmt; stmt->next; stmt = stmt->next)
			//		;
			//	stmt->sym->def_value = sym->def_value;
			//	stmt->def = sym;
			//}
			break;
		case '\n':
			break;
		default:
			continue;
		}
	}
	fclose(in);
	for (sym = symbol_hash[256]; sym; sym = sym->next) {
		struct expr *e;

		for (e = sym->stmt->dep; e; e = e->left.expr) {
			if (e->right.sym == &symbol_no)
				continue;
			if (!e->right.sym->def_value) {
				sym->def_value = NULL;
				break;
			}
			if (sym->def_value && e->right.sym->def_value == &symbol_no)
				continue;
			if (e->right.sym->def_value == &symbol_yes)
				sym->stmt->def = e->right.sym;
			sym->def_value = e->right.sym->def_value;
		}
	}
	return 0;
}

int writeconf(void)
{
	FILE *out, *out_h;
	struct statement *stmt;
	struct symbol *sym;
	struct menu *menu;
	int type;

	out = fopen(".tmpconfig", "w");
	if (!out)
		return 1;
	out_h = fopen(".tmpconfig.h", "w");
	if (!out_h)               
		return 1;
	fprintf(out, "#\n"
		     "# Automatically generated make config: don't edit\n"
		     "#\n");
	fprintf(out_h, "/*\n"
		       " * Automatically generated C config: don't edit\n"
		       " */\n"
		       "#define AUTOCONF_INCLUDED\n");

	clear_sym_deps();
	menu = rootmenu.list;
	while (menu) {
		sym = menu->sym;
		if ((stmt = menu->stmt)) {
			switch (stmt->token) {
			case T_MENU:
			case T_COMMENT:
				if (calc_value(stmt->dep) == &symbol_no)
					goto skip;
				fprintf(out, "\n"
					     "#\n"
					     "# %s\n"
					     "#\n", stmt->prompt);
				fprintf(out_h, "\n"
					       "/*\n"
					       " * %s\n"
					       " */\n", stmt->prompt);
				break;
			//case '^':
			//	calc_sym_value(sym);
			//	break;
			}
		} else if (sym) {
			calc_sym_value(sym);
			if (!(sym->flags & SYMBOL_WRITE))
				goto next;
			type = sym->type;
			if (type == SYMBOL_TRISTATE && calc_sym_value(modules_sym) == &symbol_no)
				type = SYMBOL_BOOLEAN;
			if (sym->value == &symbol_no) {
				fprintf(out, "# CONFIG_%s is not set\n", sym->name);
				fprintf(out_h, "#undef CONFIG_%s\n", sym->name);
			} else if (type == SYMBOL_BOOLEAN) {
				fprintf(out, "CONFIG_%s=y\n", sym->name);
				fprintf(out_h, "#define CONFIG_%s 1\n", sym->name);
			} else if (sym->value == &symbol_mod) {
				fprintf(out, "CONFIG_%s=m\n", sym->name);
				fprintf(out_h, "#undef CONFIG_%s\n", sym->name);
				fprintf(out_h, "#define CONFIG_%s_MODULE 1\n", sym->name);
			} else if (type == SYMBOL_STRING) {
				// fix me
				fprintf(out, "CONFIG_%s=\"%s\"\n", sym->name, sym->value->name);
				fprintf(out_h, "#define CONFIG_%s \"%s\"\n", sym->name, sym->value->name);
			} else {
				fprintf(out, "CONFIG_%s=%s\n", sym->name, sym->value->name);
				fprintf(out_h, "#define CONFIG_%s %s\n", sym->name, sym->value->name);
			}
		}

	next:
		if (menu->list) {
			menu = menu->list;
			continue;
		}
	skip:
		if (menu->next)
			menu = menu->next;
		else while ((menu = menu->parent)) {
			if (menu->next) {
				menu = menu->next;
				break;
			}
		}
	}

	rename(".config", ".config.old");
	rename(".tmpconfig", ".config");
	rename(".tmpconfig.h", "include/linux/autoconf.h");

	return 0;
}
